#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.18
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { createRequire } from 'node:module';
import { program } from 'commander';
import sh from 'shelljs';
import chalk from 'chalk';
import { getType } from 'js-cool';
import { isNeedUpgrade, upgradeGitmars, queue } from '@gitmars/core';
import { getIsGitProject, getConfig, getGitConfig, checkGitStatus, getCurrentBranch, searchBranches, getIsBranchOrCommitExist, getIsMergedTargetBranch } from '@gitmars/git';
import { createArgs } from '@gitmars/utils';
import { getUserInfo } from '@gitmars/api';
import lang from './common/local.mjs';
import { defaults } from './common/global.mjs';
import { cmdConfig } from './conf/end.mjs';

const { t } = lang;
const require = createRequire(import.meta.url);
const { args, options } = cmdConfig;
const { red } = chalk;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
const config = getConfig();
const { appName } = getGitConfig();
const mergeRequestModule = require.resolve("@gitmars/api");
program.name("gitm end").usage("[type] [name] [--description [description]] [--as-feature] [--no-combine]").description(
  t(
    "Merge bugfix task branch, merge feature function development branch, the corresponding branch will be deleted after the merge is completed"
  )
);
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (type, name, opt) => {
  const userInfoApi = config.apis?.userInfo?.url || config.api;
  const needUpgrade = await isNeedUpgrade(config.versionControlType);
  needUpgrade && upgradeGitmars();
  const allow = ["bugfix", "feature", "support"];
  const deny = [
    defaults.master,
    defaults.develop,
    defaults.release,
    defaults.bugfix,
    defaults.support
  ];
  const { level, nickname = "" } = userInfoApi ? await getUserInfo() : {};
  const status = checkGitStatus();
  let _nameArr = [], isDescriptionCorrect = true;
  if (!status)
    process.exit(1);
  if (config.descriptionValidator) {
    const reg = getType(config.descriptionValidator) === "regexp" ? config.descriptionValidator : new RegExp(config.descriptionValidator);
    isDescriptionCorrect = Boolean(opt.description && reg.test(opt.description));
  }
  if (!type) {
    [type, ..._nameArr] = getCurrentBranch().split("/");
    name = _nameArr.join("/");
    if (!name) {
      deny.includes(type) && sh.echo(
        red(
          t(
            "Hey bro, what is the fuck are you doing by executing this command in the {type} branch?",
            { type }
          )
        )
      );
      process.exit(1);
    }
  } else if (!name) {
    if (allow.includes(type)) {
      sh.echo(t("Enter branch name"));
      process.exit(1);
    }
    const branches = searchBranches({ type });
    if (branches.length === 1) {
      [type, ..._nameArr] = branches[0].split("/");
      name = _nameArr.join("/");
    } else {
      sh.echo(
        branches.length > 1 ? t(
          "If you find multiple branches with names containing {type}, please enter the branch type",
          { type }
        ) : red(t("Branch does not exist, please enter it correctly"))
      );
      process.exit(1);
    }
  }
  if (type === "feature" && opt.asFeature) {
    sh.echo(t("--as-feature is only used in the bugfix branch."));
    process.exit(1);
  }
  const isRemoteBranchExist = getIsBranchOrCommitExist(`${type}/${name}`, true);
  if (allow.includes(type) && name) {
    const base = opt.asFeature ? config.release : type === "bugfix" ? config.bugfix : config.release;
    let cmd = [];
    const isNeedCombineDevelop = !getIsMergedTargetBranch(`${type}/${name}`, config.develop, {
      remote: true
    });
    const isNeedCombineBase = !getIsMergedTargetBranch(`${type}/${name}`, base, {
      remote: true
    });
    const isNeedCombineBugfix = !getIsMergedTargetBranch(`${type}/${name}`, config.bugfix, {
      remote: true
    });
    if (opt.combine && isNeedCombineDevelop) {
      cmd = [
        "git fetch",
        `git checkout ${config.develop}`,
        "git pull",
        {
          cmd: `git merge --no-ff ${type}/${name}`,
          config: {
            again: false,
            success: t("Merge {source} into {target} successfully", {
              source: `${type}/${name}`,
              target: config.develop
            }),
            fail: t(
              "An error occurred merging {source} to {target}, Please follow the instructions",
              {
                source: `${type}/${name}`,
                target: config.develop
              }
            )
          }
        },
        {
          cmd: "git push",
          config: {
            again: true,
            success: t("Successful Pushed"),
            fail: t("Push failed, please follow the prompts")
          }
        },
        `git checkout ${type}/${name}`
      ];
    }
    if (type === "support" && opt.combine && isNeedCombineBugfix) {
      if (!level || level < 3) {
        cmd = cmd.concat([
          "git fetch",
          `git checkout ${config.bugfix}`,
          "git pull",
          {
            cmd: `git merge --no-ff ${type}/${name}`,
            config: {
              again: false,
              success: t("Merge {source} into {target} successfully", {
                source: `${type}/${name}`,
                target: config.bugfix
              }),
              fail: t(
                "An error occurred merging {source} to {target}, Please follow the instructions",
                {
                  source: `${type}/${name}`,
                  target: config.bugfix
                }
              )
            }
          },
          {
            cmd: "git push",
            config: {
              again: true,
              success: t("Successful Pushed"),
              fail: t("Push failed, please follow the prompts")
            }
          },
          `git checkout ${type}/${name}`
        ]);
      } else {
        if (!isDescriptionCorrect) {
          sh.echo(
            red(
              t(
                "The description of the reason for submission does not meet the specification"
              )
            )
          );
          process.exit(1);
        }
        cmd = cmd.concat([
          {
            cmd: `git push --set-upstream origin ${type}/${name}`,
            config: {
              again: true,
              success: t("Push remote and associate remote branch successfully"),
              fail: t("Push remote failed, please follow the prompts")
            }
          },
          {
            cmd: {
              module: mergeRequestModule,
              entry: "createMergeRequest",
              options: {
                source_branch: `${type}/${name}`,
                target_branch: config.bugfix,
                description: opt.description
              }
            },
            config: {
              again: true,
              success: t("Successfully created merge request"),
              fail: t(
                "There was an error creating the merge request, please follow the instructions"
              )
            }
          },
          [
            "gitm",
            "postmsg",
            `"${t(
              "{nickname} submitted a merge request for {source} branch to {target} branch in {app} project",
              {
                nickname,
                app: appName,
                source: `${type}/${name}`,
                target: config.bugfix
              }
            )}"`
          ]
        ]);
      }
    }
    if (!opt.combine || !isNeedCombineBase) {
      cmd = cmd.concat([
        `git checkout ${config.develop}`,
        `git branch -D ${type}/${name}`,
        {
          cmd: "git remote prune origin",
          config: {
            again: true,
            success: t("Cleanup of remote branch was successful"),
            fail: t("Failed to clean up remote branch, please follow the prompts")
          }
        }
      ]);
      if (isRemoteBranchExist) {
        cmd = cmd.concat([
          {
            cmd: `git push origin --delete ${type}/${name}`,
            config: {
              again: true,
              success: t("Successfully deleted remote branch"),
              fail: t("Deletion failed, please contact administrator")
            }
          }
        ]);
      }
    } else {
      if (!level || level < 3) {
        cmd = cmd.concat([
          "git fetch",
          `git checkout ${base}`,
          "git pull",
          {
            cmd: `git merge --no-ff ${type}/${name}`,
            config: {
              again: false,
              success: t("Merge {source} into {target} successfully", {
                source: `${type}/${name}`,
                target: base
              }),
              fail: t(
                "An error occurred merging {source} to {target}, Please follow the instructions",
                {
                  source: `${type}/${name}`,
                  target: base
                }
              )
            }
          },
          {
            cmd: "git push",
            config: {
              again: true,
              success: t("Successful Pushed"),
              fail: t("Push failed, please follow the prompts")
            }
          },
          `git checkout ${config.develop}`,
          `git branch -D ${type}/${name}`,
          {
            cmd: "git remote prune origin",
            config: {
              again: true,
              success: t("Cleanup of remote branch was successful"),
              fail: t("Failed to clean up remote branch, please follow the prompts")
            }
          }
        ]);
        if (isRemoteBranchExist) {
          cmd = cmd.concat([
            {
              cmd: `git push origin --delete ${type}/${name}`,
              config: {
                again: true,
                success: t("Successfully deleted remote branch"),
                fail: t("Deletion failed, please contact administrator")
              }
            }
          ]);
        }
      } else {
        if (!isDescriptionCorrect) {
          sh.echo(
            red(
              t(
                "The description of the reason for submission does not meet the specification"
              )
            )
          );
          process.exit(1);
        }
        cmd = cmd.concat([
          {
            cmd: `git push --set-upstream origin ${type}/${name}`,
            config: {
              again: true,
              success: t("Push remote and associate remote branch successfully"),
              fail: t("Push remote failed, please follow the prompts")
            }
          },
          {
            cmd: {
              module: mergeRequestModule,
              entry: "createMergeRequest",
              options: {
                source_branch: `${type}/${name}`,
                target_branch: base,
                description: opt.description
              }
            },
            config: {
              again: true,
              success: t("Successfully created merge request"),
              fail: t(
                "There was an error creating the merge request, please follow the instructions"
              )
            }
          },
          [
            "gitm",
            "postmsg",
            `"${t(
              "{nickname} submitted a merge request for {source} branch to {target} branch in {app} project",
              {
                nickname,
                app: appName,
                source: `${type}/${name}`,
                target: base
              }
            )}"`
          ]
        ]);
      }
    }
    queue(cmd);
  } else {
    sh.echo(red(t("type only allows input") + ": " + JSON.stringify(allow)));
    process.exit(1);
  }
});
program.parse(process.argv);
