#!/usr/bin/env node
/*!
 * gitmars v7.0.0-beta.18
 * 这是一个git工作流工具
 * (c) 2021-2024 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
import { program } from 'commander';
import dayjs from 'dayjs';
import { checkbox } from '@inquirer/prompts';
import sh from 'shelljs';
import chalk from 'chalk';
import to from 'await-to-done';
import { queue } from '@gitmars/core';
import { getIsGitProject, getCurrentBranch } from '@gitmars/git';
import { createArgs, echo } from '@gitmars/utils';
import { getRevertCache, delRevertCache } from '@gitmars/cache';
import lang from './common/local.mjs';
import { cmdConfig } from './conf/redo.mjs';

const { t } = lang;
const { blue, green, red, yellow } = chalk;
const { args, options } = cmdConfig;
if (!getIsGitProject()) {
  sh.echo(red(t("The current directory is not a git project directory")));
  process.exit(1);
}
program.name("gitm redo").usage("[commitid...] [-m --mode [mode]]").description(t("Undo a commit record"));
if (args.length > 0)
  program.arguments(createArgs(args));
options.forEach((o) => {
  program.option(o.flags, o.description, o.defaultValue);
});
program.action(async (commitid, opt) => {
  const current = getCurrentBranch();
  let revertCache = getRevertCache(current), cmd = [], mode = "";
  mode = " -m " + Math.abs(Number(opt.mode || 1));
  if (commitid.length > 0) {
    revertCache = revertCache.filter(
      (item) => commitid.some((id) => item.after["%H"].includes(id))
    );
  }
  if (revertCache.length === 0) {
    echo(yellow(t("No recoverable undo logs found, process exited")));
    process.exit(0);
  }
  const [, commitIDs = []] = await to(
    checkbox({
      message: t("Please select the undo record to restore"),
      choices: revertCache.map(({ after }, index) => {
        const _time = dayjs(after["%aI"]).format("YYYY/MM/DD HH:mm");
        return {
          name: `${green(index + 1 + ".")} ${green(after["%s"])} | ${yellow(
            after["%an"]
          )} | ${blue(_time)}`,
          value: after["%H"],
          checked: false
        };
      })
    })
  );
  if (commitIDs.length === 0) {
    echo(yellow(t("No logs selected, process has exited")));
    process.exit(0);
  }
  revertCache = revertCache.filter((item) => commitIDs.includes(item.after["%H"]));
  cmd = revertCache.map((item) => ({
    cmd: `git revert -s --no-edit ${item.after["%H"]}${mode}`,
    config: {
      again: false,
      success: t("Undo successfully: {something}", {
        something: item.after["%s"]
      }),
      fail: t("An error has occurred, please follow the instructions")
    }
  }));
  queue(cmd).then(() => {
    delRevertCache(commitIDs);
  });
});
program.parse(process.argv);
